package jp.co.yahoo.hadoop.ml.aozora;

/**
 * A utility class for parsing a book in Aozora-Bunko.
 */
public class AozoraParser {

	private AozoraParser() {
		throw new AssertionError();
	}

	/**
	 * Parse the text of a book.
	 * 
	 * @param input
	 *            input string
	 */
	public static Aozora parse(String input) {
		input = input.replaceAll("\r", "");
		String[] lines = input.split("\n");
		String title = lines[0];
		String author = normalizeAuthor(lines[1]);
		StringBuffer body = new StringBuffer(128);
		int newLineCount = 0;
		for (int i = 2; i < lines.length; i++) {
			body.append(lines[i]);
			// Delete the buffer if the line contains hyphens only.
			if (lines[i].matches("-+"))
				body.delete(0, body.length());

			if (lines[i].length() == 0) {
				newLineCount++;
			} else {
				newLineCount = 0;
			}
			// End if 3 new lines appear.
			if (newLineCount >= 3)
				break;
		}
		return new Aozora(title, author, normalizeBody(body.toString()));
	}

	private static String normalizeBody(String input) {
		return input.replaceAll("(《.*?》|［＃.*?］|〔.*?〕|｜|　)", "");
	}

	private static String normalizeAuthor(String input) {
		return input.replaceAll("(\\s| |・|　|※|、|［.*?］|[-→].*$)", "");
	}
}