package jp.co.yahoo.hadoop.ml.aozora;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.File;

/**
 * A class extracting lifetimes of authors from files.
 */
public class ExtractAuthorLife {
	public static void main(String[] args) throws Exception {
		ExtractAuthorLife eal = new ExtractAuthorLife();
		eal.run(args);
	}

	/**
	 * Main method for extracting lifetimes of authors from files.
	 * 
	 * @param args
	 *            command line arguments
	 * @throws Exception
	 */
	public void run(String[] args) throws Exception {
		File dir = new File(args[0]);
		File[] files = dir.listFiles();
		for (File file : files) {
			BufferedReader br = new BufferedReader(new InputStreamReader(
					new FileInputStream(file), "EUC-JP"));
			try {
				String line;
				String author = null;
				int birth = -1, death = -1;
				while ((line = br.readLine()) != null) {
					String tmpAuthor = extractAuthor(line);
					if (tmpAuthor != null)
						author = tmpAuthor;
					int tmpBirth = extractBirth(line);
					if (tmpBirth != -1)
						birth = tmpBirth;
					int tmpDeath = extractDeath(line);
					if (tmpDeath != -1)
						death = tmpDeath;
				}
				if (author != null && birth != -1 && death != -1) {
					int life = (int) ((death - birth) / 10000);
					System.out.println(author + "," + life);
				}
			} finally {
				br.close();
			}
		}
	}

	private String extractAuthor(String input) {
		if (input.contains("作家名：")) {
			return input.replaceAll("(<.*?>|作家名：|\\s| |・|　|※|、|［.*?］|[-→].*$)",
					"");
		}
		return null;
	}

	private int extractBirth(String input) {
		if (input.contains("生年：")) {
			String result = input.replaceAll("(<.*?>|生年：|\\s| |（.*?）)", "");
			return parseDate(result);
		}
		return -1;
	}

	private int extractDeath(String input) {
		if (input.contains("没年：")) {
			String result = input.replaceAll("(<.*?>|没年：|\\s| |（.*?）)", "");
			return parseDate(result);
		}
		return -1;
	}

	private int parseDate(String input) {
		String[] date = input.split("[-\\.年月日]");
		if (date.length > 0 && date[0].matches("^[0-9]+$")) {
			int year = Integer.parseInt(date[0]);
			int mon = 1;
			if (date.length > 1 && date[1].matches("^[0-9]+$")) {
				mon = Integer.parseInt(date[1]);
			}
			int day = 1;
			if (date.length > 2 && date[2].matches("^[0-9]+$")) {
				day = Integer.parseInt(date[2]);
			}
			String result = String.format("%04d%02d%02d", year, mon, day);
			return (int) Integer.parseInt(result);
		}
		return -1;
	}
}
