package jp.co.yahoo.hadoop.ml.classifier;

import java.io.IOException;
import java.util.Map.Entry;

import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.MapWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.VIntWritable;
import org.apache.hadoop.io.Writable;
import org.apache.hadoop.mapreduce.Reducer;

/**
 * A reducer for multi-class classification
 */
public class MulticlassClassifierReducer extends
		Reducer<VIntWritable, MapWritable, VIntWritable, MapWritable> {
	
	@Override
	public void reduce(VIntWritable key, Iterable<MapWritable> values,
			Context context) throws IOException, InterruptedException {
		MapWritable mergedMap = new MapWritable();
		int length = 0;
		for (MapWritable value : values) {
			for (Entry<Writable, Writable> entry : value.entrySet()) {
				Text word = (Text) entry.getKey();
				DoubleWritable weight = (DoubleWritable) entry.getValue();
				DoubleWritable mergedWeight = (DoubleWritable) (mergedMap
						.get(word));
				if (mergedWeight == null) {
					mergedMap.put(word, weight);
				} else {
					mergedWeight.set(mergedWeight.get() + weight.get());
				}
			}
			length++;
		}

		for (Entry<Writable, Writable> entry : mergedMap.entrySet()) {
			DoubleWritable weight = (DoubleWritable) entry.getValue();
			weight.set(weight.get() / length);
		}

		context.write(key, mergedMap);
	}
}
