package jp.co.yahoo.hadoop.ml.classifier;

import java.io.IOException;
import java.util.Map.Entry;

import jp.co.yahoo.hadoop.ml.common.CommonUtil;
import jp.co.yahoo.hadoop.ml.common.DiscriminantFunctionAlgorithm;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.MapWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.VIntWritable;
import org.apache.hadoop.io.Writable;

/**
 * A mapper for logistic regression
 */
public class MulticlassLogisticRegressionMapper extends
		MulticlassClassifierMapper {
	private double stepSize = 0.001;

	@Override
	public void setup(Context context) throws IOException, InterruptedException {
		Configuration conf = context.getConfiguration();
		stepSize = conf.getFloat("sgm.step.size", (float) stepSize);
		super.setup(context);
	}

	@Override
	public void map(VIntWritable key, MapWritable value, Context context)
			throws IOException, InterruptedException {
		int b = key.get();
		//System.out.println("correctOld: " + b);
		value.put(new Text(CommonUtil.BIAS_KEY), new VIntWritable(1));

		double[] y = new double[weightMap.size()];
		double ySum = 0;
		double yMax = 0;
		double wxMax = 0;

		for (VIntWritable cat : weightMap.keySet()) {
			double wx = DiscriminantFunctionAlgorithm.predict(value,
					weightMap.get(cat));
			y[cat.get()] = wx;
			wxMax = Math.max(wxMax, wx);
		}

		for (VIntWritable cat : weightMap.keySet()) {
			double yk = Math.exp(y[cat.get()] - wxMax);
			y[cat.get()] = yk;
			ySum += yk;
			yMax = Math.max(yMax, yk);
		}

		for (VIntWritable cat : weightMap.keySet()) {
			MapWritable weightMapPerCat = weightMap.get(cat);
			int catInt = cat.get();
			int bk = ((int) b / classWidth == catInt) ? 1 : 0;
			double prob = y[catInt] / ySum;

			if (y[catInt] == yMax) {
				System.out.println("maxCat: " + (catInt * classWidth) + "("
						+ prob + ")");
			}
			if (bk == 1) {
				System.out.println("correctCat: " + b + "("
						+ prob + ")");
			}

			for (Entry<Writable, Writable> entry : value.entrySet()) {
				Text word = (Text) entry.getKey();
				int x = ((VIntWritable) entry.getValue()).get();
				DoubleWritable weightWritable = (DoubleWritable) weightMapPerCat
						.get(word);
				double diffWeight = -stepSize * (prob - bk) * x;
				if (weightWritable == null) {
					weightMapPerCat.put(word, new DoubleWritable(diffWeight));
				} else {
					double w = weightWritable.get() + diffWeight;
					weightWritable.set(w);
				}
			}
		}
		
		System.out.println();
	}
}
