package jp.co.yahoo.hadoop.ml.classifier;

import jp.co.yahoo.hadoop.ml.common.TestMAEReducer;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.input.SequenceFileInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.hadoop.mapreduce.lib.output.TextOutputFormat;
import org.apache.hadoop.util.GenericOptionsParser;

/**
 * A MapReduce driver for the test of multi-class classification
 */
public class TestMulticlassClassifierDriver {
	public static void main(String[] args) throws Exception {
		Configuration conf = new Configuration();

		String[] otherArgs = new GenericOptionsParser(conf, args)
				.getRemainingArgs();
		if (otherArgs.length < 3) {
			System.err
					.println("Usage: hadoop <input file> <weight file> <output file> [algorithm]");
			System.exit(2);
		}

		conf.set("weight.file", otherArgs[1]);
		if (otherArgs.length > 3)
			conf.set("test.algorithm", otherArgs[3]);

		Job job = new Job(conf, "test multiclass");
		job.setJarByClass(TestMulticlassClassifierDriver.class);
		job.setMapperClass(TestMulticlassClassifierMapper.class);
		job.setReducerClass(TestMAEReducer.class);
		job.setInputFormatClass(SequenceFileInputFormat.class);
		job.setOutputFormatClass(TextOutputFormat.class);
		job.setMapOutputKeyClass(IntWritable.class);
		job.setMapOutputValueClass(DoubleWritable.class);
		job.setOutputKeyClass(NullWritable.class);
		job.setOutputValueClass(Text.class);
		job.setNumReduceTasks(1);
		FileInputFormat.addInputPath(job, new Path(otherArgs[0]));
		FileOutputFormat.setOutputPath(job, new Path(otherArgs[2]));
		System.exit(job.waitForCompletion(true) ? 0 : 1);
	}
}