package jp.co.yahoo.hadoop.ml.regression;

import java.io.IOException;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.MapWritable;
import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.input.SequenceFileInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.hadoop.mapreduce.lib.output.SequenceFileOutputFormat;

import jp.co.yahoo.hadoop.ml.common.IterativeParameterMixingDriver;

/**
 * A MapReduce driver for regression using Iterative Parameter Mixing
 */
public class RegressionIterativeParameterMixingDriver extends
		IterativeParameterMixingDriver {
	
	public static void main(String[] args) throws Exception {
		RegressionIterativeParameterMixingDriver driver = new RegressionIterativeParameterMixingDriver();
		driver.run(args);
	}

	@Override
	protected void runIteration(Configuration conf, Path input, Path output,
			String algorithm) throws IOException, InterruptedException,
			ClassNotFoundException {
		Job job = new Job(conf, algorithm + ": " + output);
		job.setJarByClass(IterativeParameterMixingDriver.class);
		job.setMapperClass(getMapperClass(algorithm));
		job.setReducerClass(RegressionReducer.class);
		job.setInputFormatClass(SequenceFileInputFormat.class);
		job.setOutputFormatClass(SequenceFileOutputFormat.class);
		job.setMapOutputKeyClass(DoubleWritable.class);
		job.setOutputKeyClass(NullWritable.class);
		job.setOutputValueClass(MapWritable.class);
		job.setNumReduceTasks(1);
		FileInputFormat.addInputPath(job, input);
		FileOutputFormat.setOutputPath(job, output);

		if (!job.waitForCompletion(true)) {
			throw new InterruptedException("job failed: " + output);
		}
	}

	private Class<? extends Mapper> getMapperClass(String algorithm) {
		Class<? extends Mapper> resultClass = LeastSquaresMapper.class;
		if (algorithm.equals("pa")) {
			resultClass = PassiveAggressiveMapper.class;
		}
		return resultClass;
	}
}
