package jp.co.yahoo.hadoop.ml.regression;

import java.io.IOException;

import jp.co.yahoo.hadoop.ml.common.CommonUtil;
import jp.co.yahoo.hadoop.ml.common.DiscriminantFunctionAlgorithm;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.MapWritable;
import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.io.SequenceFile;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.VIntWritable;
import org.apache.hadoop.mapreduce.Mapper;

/**
 * A mapper for the test of regression
 */
public class TestRegressionMapper extends Mapper<VIntWritable, MapWritable, IntWritable, DoubleWritable>{
	private String algorithm = "liner";
	private MapWritable weightMap = new MapWritable();
	private int trainSize = 0;
	private double diffSum = 0;
	private double multiplySize = CommonUtil.MAX_AGE;
	
	@Override
	public void setup(Context context) throws IOException, InterruptedException {
		Configuration conf = context.getConfiguration();
		multiplySize = conf.getFloat("multiply.size", (float)multiplySize);
		algorithm = conf.get("test.algorithm", algorithm);
		String weightFile = conf.get("weight.file");
		if (weightFile != null) {
			FileSystem fs = FileSystem.get(conf);
			SequenceFile.Reader reader = new SequenceFile.Reader(fs, new Path(
					weightFile), conf);
			try {
				NullWritable key = NullWritable.get();
				reader.next(key, weightMap);
			} finally {
				reader.close();
			}
		}
	}
	
	@Override
	public void map(VIntWritable key, MapWritable value, Context context)
			throws IOException, InterruptedException {
		int b = key.get();
		value.put(new Text(CommonUtil.BIAS_KEY), new VIntWritable(1));
		double wx = 0;
		if (algorithm.equals("log")){
			wx = DiscriminantFunctionAlgorithm
			.predictLogistic(value, weightMap, multiplySize);
		}else{
			wx = DiscriminantFunctionAlgorithm
				.predict(value, weightMap);
		}
		if (wx > multiplySize)
			wx = multiplySize;
		if (wx < 0)
			wx = 0;
		diffSum += Math.abs(wx - b);
		trainSize++;
	}
	
	@Override
	public void cleanup(Context context) throws IOException,
			InterruptedException {
		context.write(new IntWritable(trainSize), new DoubleWritable(diffSum));
	}
}